@@ -1,5 +1,8 @@
 /*
- * regular read/write sync io engine
+ * sync engine
+ *
+ * IO engine that does regular read(2)/write(2) with lseek(2) to transfer
+ * data.
  *
  */
 #include <stdio.h>
@@ -60,6 +63,8 @@ static struct ioengine_ops ioengine = {
 	.version	= FIO_IOOPS_VERSION,
 	.prep		= fio_syncio_prep,
 	.queue		= fio_syncio_queue,
+	.open_file	= generic_open_file,
+	.close_file	= generic_close_file,
 	.flags		= FIO_SYNCIO,
 };
 
