@@ -11,8 +11,10 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <getopt.h>
 
 #include "list.h"
+#include "rbtree.h"
 #include "md5.h"
 #include "crc32.h"
 #include "arch.h"
@@ -23,6 +25,10 @@
 #include "syslet.h"
 #endif
 
+#ifdef FIO_HAVE_GUASI
+#include <guasi.h>
+#endif
+
 enum fio_ddir {
 	DDIR_READ = 0,
 	DDIR_WRITE,
@@ -73,7 +79,10 @@ struct io_log {
  * When logging io actions, this matches a single sent io_u
  */
 struct io_piece {
-	struct list_head list;
+	union {
+		struct rb_node rb_node;
+		struct list_head list;
+	};
 	struct fio_file *file;
 	unsigned long long offset;
 	unsigned long len;
@@ -93,6 +102,8 @@ enum {
 	IO_U_F_FLIGHT	= 1 << 1,
 };
 
+struct thread_data;
+
 /*
  * The io unit
  */
@@ -109,6 +120,9 @@ struct io_u {
 #endif
 #ifdef FIO_HAVE_SYSLET
 		struct syslet_req req;
+#endif
+#ifdef FIO_HAVE_GUASI
+		guasi_req_t greq;
 #endif
 	};
 	struct timeval start_time;
@@ -150,7 +164,7 @@ struct io_u {
 	/*
 	 * Callback for io completion
 	 */
-	int (*end_io)(struct io_u *);
+	int (*end_io)(struct thread_data *, struct io_u *);
 };
 
 /*
@@ -168,6 +182,7 @@ enum {
 	VERIFY_NONE = 0,		/* no verification */
 	VERIFY_MD5,			/* md5 sum data blocks */
 	VERIFY_CRC32,			/* crc32 sum data blocks */
+	VERIFY_NULL,			/* pretend to verify */
 };
 
 /*
@@ -222,7 +237,11 @@ enum fio_file_flags {
 	FIO_FILE_OPEN		= 1 << 0,	/* file is open */
 	FIO_FILE_UNLINK		= 1 << 1,	/* unlink on close */
 	FIO_FILE_CLOSING	= 1 << 2,	/* file being closed */
-	FIO_FILE_EXISTS		= 1 << 3,	/* no need to create */
+	FIO_FILE_EXISTS		= 1 << 3,	/* file there */
+	FIO_FILE_EXTEND		= 1 << 4,	/* needs extend */
+	FIO_FILE_NOSORT		= 1 << 5,	/* don't sort verify blocks */
+	FIO_FILE_DONE		= 1 << 6,	/* io completed to this file */
+	FIO_SIZE_KNOWN		= 1 << 7,	/* size has been set */
 };
 
 /*
@@ -239,11 +258,20 @@ struct fio_file {
 		unsigned long file_data;
 		int fd;
 	};
+
+	/*
+	 * filename and possible memory mapping
+	 */
 	char *file_name;
 	void *mmap;
-	unsigned long long file_size;
+
+	/*
+	 * size of the file, offset into file, and io size from that offset
+	 */
 	unsigned long long real_file_size;
 	unsigned long long file_offset;
+	unsigned long long io_size;
+
 	unsigned long long last_pos;
 	unsigned long long last_completed_pos;
 
@@ -308,39 +336,31 @@ struct thread_stat {
 	unsigned long total_run_time;
 };
 
-/*
- * This describes a single thread/process executing a fio job.
- */
-struct thread_data {
+struct thread_options {
 	int pad;
 	char *description;
 	char *name;
 	char *directory;
 	char *filename;
+	char *opendir;
 	char *ioengine;
-	char verror[128];
-	pthread_t thread;
-	int thread_number;
-	int groupid;
-	struct thread_stat ts;
-	struct fio_file *files;
-	unsigned int files_index;
-	unsigned int nr_files;
-	unsigned int nr_open_files;
-	unsigned int nr_normal_files;
-	union {
-		unsigned int next_file;
-		os_random_state_t next_file_state;
-	};
-	int error;
-	pid_t pid;
-	char *orig_buffer;
-	size_t orig_buffer_size;
-	volatile int terminate;
-	volatile int runstate;
 	enum td_ddir td_ddir;
-	unsigned int ioprio;
-	unsigned int last_was_sync;
+	unsigned int ddir_nr;
+	unsigned int iodepth;
+	unsigned int iodepth_low;
+	unsigned int iodepth_batch;
+
+	unsigned long long size;
+	unsigned long long file_size_low;
+	unsigned long long file_size_high;
+	unsigned long long start_offset;
+
+	unsigned int bs[2];
+	unsigned int min_bs[2];
+	unsigned int max_bs[2];
+
+	unsigned int nr_files;
+	unsigned int open_files;
 
 	unsigned int odirect;
 	unsigned int invalidate_cache;
@@ -349,6 +369,7 @@ struct thread_data {
 	unsigned int end_fsync;
 	unsigned int sync_io;
 	unsigned int verify;
+	unsigned int verifysort;
 	unsigned int use_thread;
 	unsigned int unlink;
 	unsigned int do_disk_util;
@@ -360,9 +381,6 @@ struct thread_data {
 	unsigned int bs_unaligned;
 	unsigned int fsync_on_close;
 
-	unsigned int bs[2];
-	unsigned int min_bs[2];
-	unsigned int max_bs[2];
 	unsigned int hugepage_size;
 	unsigned int rw_min_bs;
 	unsigned int thinktime;
@@ -377,32 +395,80 @@ struct thread_data {
 	unsigned long long zone_size;
 	unsigned long long zone_skip;
 	enum fio_memtype mem_type;
-	char *mmapfile;
-	int mmapfd;
+
 	unsigned int stonewall;
+	unsigned int new_group;
 	unsigned int numjobs;
-	unsigned int iodepth;
-	unsigned int iodepth_low;
-	unsigned int iodepth_batch;
 	os_cpu_mask_t cpumask;
 	unsigned int iolog;
 	unsigned int read_iolog;
 	unsigned int rwmixcycle;
-	unsigned int rwmixread;
-	unsigned int rwmixwrite;
+	unsigned int rwmix[2];
 	unsigned int nice;
 	unsigned int file_service_type;
 	unsigned int group_reporting;
-	unsigned int open_files;
-	char *opendir;
+	unsigned int fadvise_hint;
 
 	char *read_iolog_file;
 	char *write_iolog_file;
+
+	/*
+	 * Pre-run and post-run shell
+	 */
+	char *exec_prerun;
+	char *exec_postrun;
+
+	unsigned int rate;
+	unsigned int ratemin;
+	unsigned int ratecycle;
+	unsigned int rate_iops;
+	unsigned int rate_iops_min;
+
+	char *ioscheduler;
+
+	/*
+	 * CPU "io" cycle burner
+	 */
+	unsigned int cpuload;
+	unsigned int cpucycle;
+};
+
+#define FIO_VERROR_SIZE	128
+
+/*
+ * This describes a single thread/process executing a fio job.
+ */
+struct thread_data {
+	struct thread_options o;
+	char verror[FIO_VERROR_SIZE];
+	pthread_t thread;
+	int thread_number;
+	int groupid;
+	struct thread_stat ts;
+	struct fio_file *files;
+	unsigned int files_index;
+	unsigned int nr_open_files;
+	unsigned int nr_normal_files;
+	union {
+		unsigned int next_file;
+		os_random_state_t next_file_state;
+	};
+	int error;
+	pid_t pid;
+	char *orig_buffer;
+	size_t orig_buffer_size;
+	volatile int terminate;
+	volatile int runstate;
+	unsigned int ioprio;
+	unsigned int last_was_sync;
+
+	char *mmapfile;
+	int mmapfd;
+
 	void *iolog_buf;
 	FILE *iolog_f;
 
 	char *sysfs_root;
-	char *ioscheduler;
 
 	os_random_state_t bsrange_state;
 	os_random_state_t verify_state;
@@ -427,17 +493,12 @@ struct thread_data {
 	/*
 	 * Rate state
 	 */
-	unsigned int rate;
-	unsigned int ratemin;
-	unsigned int ratecycle;
 	unsigned long rate_usec_cycle;
 	long rate_pending_usleep;
 	unsigned long rate_bytes;
+	unsigned long rate_blocks;
 	struct timeval lastrate;
 
-	unsigned long long io_size;
-	unsigned long long total_file_size;
-	unsigned long long start_offset;
 	unsigned long long total_io_size;
 
 	unsigned long io_issues[2];
@@ -452,32 +513,30 @@ struct thread_data {
 	 */
 	os_random_state_t random_state;
 
-	/*
-	 * CPU "io" cycle burner
-	 */
-	unsigned int cpuload;
-	unsigned int cpucycle;
-
 	struct timeval start;	/* start of this loop */
 	struct timeval epoch;	/* time job was started */
+	struct timeval rw_end[2];
+	unsigned int rw_end_set[2];
 
 	/*
 	 * read/write mixed workload state
 	 */
 	os_random_state_t rwmix_state;
+	unsigned long long rwmix_bytes;
 	struct timeval rwmix_switch;
 	enum fio_ddir rwmix_ddir;
+	unsigned int ddir_nr;
 
 	/*
-	 * Pre-run and post-run shell
-	 */
-	char *exec_prerun;
-	char *exec_postrun;
-
-	/*
-	 * IO historic logs
+	 * IO history logs for verification. We use a tree for sorting,
+	 * if we are overwriting. Otherwise just use a fifo.
 	 */
+	struct rb_root io_hist_tree;
 	struct list_head io_hist_list;
+
+	/*
+	 * For IO replaying
+	 */
 	struct list_head io_log_list;
 
 	/*
@@ -492,6 +551,11 @@ struct thread_data {
 	unsigned int file_service_nr;
 	unsigned int file_service_left;
 	struct fio_file *file_service_file;
+
+	/*
+	 * For generating file sizes
+	 */
+	os_random_state_t file_size_state;
 };
 
 /*
@@ -538,13 +602,13 @@ extern unsigned long page_mask, page_size;
 
 extern struct thread_data *threads;
 
-#define td_read(td)		((td)->td_ddir & TD_DDIR_READ)
-#define td_write(td)		((td)->td_ddir & TD_DDIR_WRITE)
-#define td_rw(td)		(((td)->td_ddir & TD_DDIR_RW) == TD_DDIR_RW)
-#define td_random(td)		((td)->td_ddir & TD_DDIR_RAND)
+#define td_read(td)		((td)->o.td_ddir & TD_DDIR_READ)
+#define td_write(td)		((td)->o.td_ddir & TD_DDIR_WRITE)
+#define td_rw(td)		(((td)->o.td_ddir & TD_DDIR_RW) == TD_DDIR_RW)
+#define td_random(td)		((td)->o.td_ddir & TD_DDIR_RAND)
 
 #define BLOCKS_PER_MAP		(8 * sizeof(long))
-#define TO_MAP_BLOCK(td, f, b)	((b) - ((f)->file_offset / (td)->rw_min_bs))
+#define TO_MAP_BLOCK(td, f, b)	((b) - ((f)->file_offset / (td)->o.rw_min_bs))
 #define RAND_MAP_IDX(td, f, b)	(TO_MAP_BLOCK(td, f, b) / BLOCKS_PER_MAP)
 #define RAND_MAP_BIT(td, f, b)	(TO_MAP_BLOCK(td, f, b) & (BLOCKS_PER_MAP - 1))
 
@@ -554,9 +618,9 @@ static inline int should_fsync(struct thread_data *td)
 {
 	if (td->last_was_sync)
 		return 0;
-	if (td->odirect)
+	if (td->o.odirect)
 		return 0;
-	if (td_write(td) || td_rw(td) || td->override_sync)
+	if (td_write(td) || td_rw(td) || td->o.override_sync)
 		return 1;
 
 	return 0;
@@ -624,7 +688,6 @@ extern void disk_util_timer_arm(void);
 extern void setup_log(struct io_log **);
 extern void finish_log(struct thread_data *, struct io_log *, const char *);
 extern void __finish_log(struct io_log *, const char *);
-extern int setup_rate(struct thread_data *);
 extern struct io_log *agg_io_log[2];
 extern int write_bw_log;
 extern void add_agg_sample(unsigned long, enum fio_ddir);
@@ -646,10 +709,18 @@ extern void fio_gettime(struct timeval *, void *);
 extern void set_genesis_time(void);
 
 /*
- * Init functions
+ * Init/option functions
  */
 extern int __must_check parse_options(int, char **);
-extern int __must_check init_random_state(struct thread_data *);
+extern int fio_option_parse(struct thread_data *, const char *);
+extern int fio_cmd_option_parse(struct thread_data *, const char *, char *);
+extern void fio_fill_default_options(struct thread_data *);
+extern int fio_show_option_help(const char *);
+extern void fio_options_dup_and_init(struct option *);
+extern void options_mem_dupe(struct thread_data *);
+extern void options_mem_free(struct thread_data *);
+#define FIO_GETOPT_JOB		0x89988998
+#define FIO_NR_OPTIONS		128
 
 /*
  * File setup/shutdown
@@ -664,6 +735,8 @@ extern void add_file(struct thread_data *, const char *);
 extern void get_file(struct fio_file *);
 extern void put_file(struct thread_data *, struct fio_file *);
 extern int add_dir_files(struct thread_data *, const char *);
+extern int init_random_map(struct thread_data *);
+extern void dup_files(struct thread_data *, struct thread_data *);
 
 /*
  * ETA/status stuff
@@ -693,7 +766,7 @@ enum {
  */
 extern void populate_verify_io_u(struct thread_data *, struct io_u *);
 extern int __must_check get_next_verify(struct thread_data *td, struct io_u *);
-extern int __must_check verify_io_u(struct io_u *);
+extern int __must_check verify_io_u(struct thread_data *, struct io_u *);
 
 /*
  * Memory helpers
@@ -740,6 +813,8 @@ extern void td_io_close_file(struct thread_data *, struct fio_file *);
 		fprintf(stderr, ##args);	\
 	} while (0)
 
+#define log_info(args...)	fprintf(f_out, ##args)
+
 FILE *get_f_out(void);
 FILE *get_f_err(void);
 
@@ -763,7 +838,7 @@ struct ioengine_ops {
 	void *dlhandle;
 };
 
-#define FIO_IOOPS_VERSION	6
+#define FIO_IOOPS_VERSION	7
 
 extern struct ioengine_ops *load_ioengine(struct thread_data *, const char *);
 extern void register_ioengine(struct ioengine_ops *);
@@ -780,7 +855,7 @@ extern void close_ioengine(struct thread_data *);
 #define for_each_td(td, i)	\
 	for ((i) = 0, (td) = &threads[0]; (i) < (int) thread_number; (i)++, (td)++)
 #define for_each_file(td, f, i)	\
-	for ((i) = 0, (f) = &(td)->files[0]; (i) < (td)->nr_files; (i)++, (f)++)
+	for ((i) = 0, (f) = &(td)->files[0]; (i) < (td)->o.nr_files; (i)++, (f)++)
 
 #define fio_assert(td, cond)	do {	\
 	if (!(cond)) {			\
@@ -792,4 +867,10 @@ extern void close_ioengine(struct thread_data *);
 	}	\
 } while (0)
 
+static inline void clear_error(struct thread_data *td)
+{
+	td->error = 0;
+	td->verror[0] = '\0';
+}
+
 #endif
